require 'rails_helper'

RSpec.describe Hermes::Entity, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:entity)).to be_valid
  end

  context 'with basic entity' do
    let!(:basic_entity) { create(:entity, siret: 'hquzs7U6v5') }

    describe 'triggered side effects are valid' do
      it 'has a group with valid name' do
        entity_ref = basic_entity.siren || basic_entity.siret
        expect(basic_entity.group.name).to eq "entity_#{entity_ref}"
        expect(basic_entity.readers).to include "g_#{basic_entity.group.id}"
      end
    end
  end

  context 'with essential items for priority points' do
    let!(:entity1) { create(:entity, siret: '123') }

    it 'has entity1 with annual stock 8500' do
      expect(entity1.annual_stock).to be 8500
    end

    describe 'entity1 priority points attribution' do
      single_request_attributed_points = [30, 40]
      request_nb = 2

      request_nb.times do |i|
        let!(:"applicant#{i}") { create(:applicant, main_employer_siret: '123') }
        let!(:"request#{i}") do
          create(
            :housing_request,
            is_active: true,
            main_applicant: send("applicant#{i}"),
            entity_validated: true,
            priority_points_indexes: single_request_attributed_points
          )
        end
      end

      it 'updates entity1 points distribution' do
        spent_points = request_nb * single_request_attributed_points.sum
        r_entity1 = entity1.reload
        expect(r_entity1.total_spent).to be(spent_points)
        expect(r_entity1.spent_on_ongoing_requests).to be(spent_points)
        expect(r_entity1.points_balance).to be(8500 - spent_points)
        expect(r_entity1.spent_on_completed_requests).to be(0).or(be nil)
        expect(r_entity1.spent_on_other).to be(0).or(be nil)
      end
    end
  end
end
