require 'rails_helper'

RSpec.describe Hermes::Lessor, type: :model do
  it 'has a valid factory' do
    expect(FactoryBot.build(:lessor)).to be_valid
  end
  
  # describe 'validate title presence' do
  #   it 'is invalid without a title' do
  #     expect(FactoryBot.build(:lessor, title: nil)).not_to be_valid
  #   end
  # end
  
  let!(:basic_lessor) {create(:lessor)}
  let!(:agency) {create(:entity)}
  let!(:lessor_objective1) do
    create(
        :lessor_objective,
        department: 'Yvelines',
        threshold_type: 'PLUS',
        priority_public: true,
       
        target_by_typologies: {T2: 30},
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"1"
    )
  end
  let!(:lessor_objective2) do
    create(
        :lessor_objective,
        department: 'Yvelines',
        priority_public: false,
       
        target_by_typologies: {T2: 30},
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"2"
    )
  end
  let!(:lessor_objective3) do
    create(
        :lessor_objective,
        district: 'Plaisir',
        department: 'Yvelines',
        threshold_type: 'PLUS',
        priority_public: true,
       
        target_by_typologies: {all: 20},
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"3"
    )
  end
  let!(:lessor_objective4) do
    create(
        :lessor_objective,
        district: 'Plaisir',
        department: 'Yvelines',
        priority_public: true,
       
        target_by_typologies: {T1: 20},
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"4"
    )
  end
  let!(:lessor_objective5) do
    create(
        :lessor_objective,
        district: 'Plaisir',
        department: 'Yvelines',
        priority_public: true,
       
        target_by_typologies: {all: 10},
        operation_name: "Bahamas",
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"5"
    )
  end
  let!(:lessor_objective6) do
    create(
        :lessor_objective,
        district: 'Plaisir',
        department: 'Yvelines',
        threshold_type: 'PLUS',
        priority_public: true,
       
        target_by_typologies: {all: 10},
        operation_name: "Bahamas",
        lessor_id: basic_lessor.id,
        activation_date: DateTime.new(DateTime.current.year, 5, 5).utc,
        agency_id: agency.id,
        test:"6"
    )
  end
  
  describe "logement A ->5" do
    let!(:housing_offerA) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          threshold_type: 'PLI',
          typology:"T5",
          priority_public: true,
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerA.lessor_status= 'LOUE'
      housing_offerA.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>1})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement B ->no_obj" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Rambouillet',
          department: 'Yvelines',
          priority_public: false,
          typology:"T5",
          threshold_type: 'PLUS',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement C ->6" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: true,
          typology:"T5",
          threshold_type: 'PLUS',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>1})
    end
  
  end
  describe "logement D ->no obj" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Rambouillet',
          department: 'Yvelines',
          priority_public: true,
          typology:"T2",
          operation_name: "autre",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement E ->no_obj" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Paris',
          department: 'PARIS',
          priority_public: true,
          typology:"T5",
          threshold_type: 'PLUS',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement F ->1" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Rambouillet',
          department: 'Yvelines',
          priority_public: true,
          typology:"T2",
          threshold_type: 'PLUS',
          operation_name: "autre",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>1})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement G ->4" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: true,
          typology:"T1",
          operation_name: "autres",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>1})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement H ->2" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: false,
          typology:"T2",
          operation_name: "autres",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>1})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement i ->no_obj" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: false,
          typology:"T3",
          threshold_type: 'PLUS',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement j ->2" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: false,
          typology:"T2",
          threshold_type: 'PLUS',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>1})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  describe "logement K ->5" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: true,
          typology:"T2",
          threshold_type: 'PLI',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>1})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  
  end
  
  describe "logement L ->no_obj" do
    let!(:housing_offerL) do
      create(
          :housing_offer,
          district: 'Plaisir',
          department: 'Yvelines',
          priority_public: false,
          typology:"T1",
          threshold_type: 'PLI',
          operation_name: "Bahamas",
          lessor_id: basic_lessor.id,
          agency_id: agency.id
      )
    end
    before :each do
      # housing_request1.applicant_ids = [applicant2.id]
      housing_offerL.lessor_status= 'LOUE'
      housing_offerL.save()
    end
    it 'should has updated correctly' do
      expect(lessor_objective1.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective2.reload.realized_by_typologies).to eq({"T2"=>0})
      expect(lessor_objective3.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective4.reload.realized_by_typologies).to eq({"T1"=>0})
      expect(lessor_objective5.reload.realized_by_typologies).to eq({"all"=>0})
      expect(lessor_objective6.reload.realized_by_typologies).to eq({"all"=>0})
    end
  end
end
